﻿using FlowerShopBusinessLogic.MailWorker;
using FlowerShopContracts.BindingModels;
using FlowerShopContracts.BusinessLogicsContracts;
using FlowerShopContracts.SearchModels;
using FlowerShopContracts.StoragesContracts;
using FlowerShopContracts.ViewModels;
using FlowerShopDataModels.Enums;
using Microsoft.Extensions.Logging;

namespace FlowerShopBusinessLogic.BusinessLogics
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
        private readonly AbstractMailWorker _mailWorker;
        private readonly IClientLogic _clientLogic;

        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage, AbstractMailWorker mailWorker, IClientLogic clientLogic)
        {
            _logger = logger;
            _orderStorage = orderStorage;
            _mailWorker = mailWorker;
            _clientLogic = clientLogic;
        }

        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id: {Id}", model?.Id);

            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }

            _logger.LogInformation("ReadList. Count: {Count}", list.Count);
            return list;
        }

        public OrderViewModel? ReadElement(OrderSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            _logger.LogInformation("ReadElement. Id:{ Id}", model.Id);
           
            var element = _orderStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }

            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            
            return element;
        }

        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);

            if (model.Status != OrderStatus.Неизвестен)
            {
                _logger.LogWarning("Invalid order status");
                return false;
            }

            model.Status = OrderStatus.Принят;

            var result = _orderStorage.Insert(model);
            if (result == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }

            SendOrderStatusMail(result.ClientId, $"Новый заказ создан. Номер заказа #{result.Id}", $"Заказ #{result.Id} от {result.DateCreate} на сумму {result.Sum:0.00} принят");

            return true;
        }

        public bool TakeOrderInWork(OrderBindingModel model)
        {
            CheckModel(model, false);

            if (_orderStorage.GetElement(new OrderSearchModel { Id=model.Id })?.Status != OrderStatus.Принят)
            {
                _logger.LogWarning("Invalid order status");
                return false;
            }

            model.Status = OrderStatus.Выполняется;

            if (!model.ImplementerId.HasValue)
            {
                _logger.LogWarning("There must be implementer to take order in work");
                return false;
            }

            var result = _orderStorage.Update(model);
            if (result == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }

            SendOrderStatusMail(result.ClientId, $"Изменен статус заказа #{result.Id}", $"Заказ #{result.Id} изменен статус на {result.Status}");

            return true;
        }
        public bool FinishOrder(OrderBindingModel model)
        {
            CheckModel(model, false);

            if (_orderStorage.GetElement(new OrderSearchModel { Id = model.Id })?.Status != OrderStatus.Выполняется)
            {
                _logger.LogWarning("Invalid order status");
                return false;
            }

            model.Status = OrderStatus.Готов;
            model.DateImplement = DateTime.Now;

            var result = _orderStorage.Update(model);
            if (result == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }

            SendOrderStatusMail(result.ClientId, $"Изменен статус заказа #{result.Id}", $"Заказ #{result.Id} изменен статус на {result.Status}");

            return true;
        }

        public bool DeliveryOrder(OrderBindingModel model)
        {
            CheckModel(model, false);

            var order = _orderStorage.GetElement(new OrderSearchModel { Id = model.Id });

            if (order?.Status != OrderStatus.Готов)
            {
                _logger.LogWarning("Invalid order status");
                return false;
            }

            model.Status = OrderStatus.Выдан;
            model.DateImplement = order.DateImplement;

            var result = _orderStorage.Update(model);
            if (result == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }

            SendOrderStatusMail(result.ClientId, $"Изменен статус заказа #{result.Id}", $"Заказ #{result.Id} изменен статус на {result.Status}");

            return true;
        }

        private bool SendOrderStatusMail(int clientId, string subject, string text)
        {
            var client = _clientLogic.ReadElement(new() { Id = clientId });
            
            if (client == null)
            {
                return false;
            }

            _mailWorker.MailSendAsync(new()
            {
                MailAddress = client.Email,
                Subject = subject,
                Text = text
            });

            return true;
        }

        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            if (!withParams)
            {
                return;
            }

            if (model.Sum <= 0)
            {
                throw new ArgumentNullException("Стоимость должна быть больше 0", nameof(model.Sum));
            }

            _logger.LogInformation("Order. Id: {Id}. Sum: {Sum}. BouquetId: {BouquetId}", model.Id, model.Sum, model.BouquetId);

            var element = _orderStorage.GetElement(new OrderSearchModel { Id = model.Id });
            if (element != null && element.Id == model.Id)
            {
                throw new InvalidOperationException("Заказ с таким идентификатором уже есть");
            }
        }
    }
}
